<?php

// Include necessary Elementor classes
use Elementor\Widget_Base;
use Elementor\Controls_Manager;

class Text_Shuffle_Widget extends Widget_Base {

    public function get_name() {
        return 'text_shuffle_widget';
    }

    public function get_title() {
        return __('Text Shuffle', 'text-shuffle-widget');
    }

    public function get_icon() {
        return 'eicon-editor-code';
    }

    public function get_categories() {
        return ['basic'];
    }

    protected function _register_controls() {
        $this->start_controls_section(
            'content_section',
            [
                'label' => __('Content', 'text-shuffle-widget'),
                'tab' => Controls_Manager::TAB_CONTENT,
            ]
        );

        // Add controls for each shortcode option
        $this->add_control(
            'text',
            [
                'label' => __('Text', 'text-shuffle-widget'),
                'type' => Controls_Manager::TEXT,
                'default' => '',
            ]
        );

        // Add controls for each shortcode option
        $this->add_control(
            'trigger_class',
            [
                'label' => __('Trigger class', 'text-shuffle-widget'),
                'type' => Controls_Manager::TEXT,
                'default' => '',
            ]
        );
		
        $this->add_control(
            'element',
            [
                'label' => __('Element', 'text-shuffle-widget'),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'h1' => __('H1', 'text-shuffle-widget'),
                    'h2' => __('H2', 'text-shuffle-widget'),
                    'h3' => __('H3', 'text-shuffle-widget'),
                    'h4' => __('H4', 'text-shuffle-widget'),
                    'h5' => __('H5', 'text-shuffle-widget'),
                    'h6' => __('H6', 'text-shuffle-widget'),
                    'span' => __('Span', 'text-shuffle-widget'),
                    'div' => __('Div', 'text-shuffle-widget'),
                    // Add more element options if needed
                ],
                'default' => 'span',
            ]
        );

		// Add alignment control
        $this->add_responsive_control(
            'alignment',
            [
                'label' => __('Alignment', 'text-shuffle-widget'),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => __('Left', 'text-shuffle-widget'),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => __('Center', 'text-shuffle-widget'),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => __('Right', 'text-shuffle-widget'),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container' => 'text-align: {{VALUE}};',
                ],
                'default' => 'left',
            ]
        );

        $this->add_control(
            'counter',
            [
                'label' => __('Amount of shuffles', 'text-shuffle-widget'),
                'type' => Controls_Manager::NUMBER,
                'default' => 3,
            ]
        );

        $this->add_control(
            'timeout',
            [
                'label' => __('Timeout between each shuffle', 'text-shuffle-widget'),
                'type' => Controls_Manager::NUMBER,
                'default' => 80,
            ]
        );

        $this->add_control(
            'trigger_distance',
            [
                'label' => __('Trigger Distance', 'text-shuffle-widget'),
                'type' => Controls_Manager::NUMBER,
                'default' => 50,
            ]
        );

		$this->end_controls_section();
		
		$this->start_controls_section(
            'style_section',
            [
                'label' => __('Style', 'text-shuffle-widget'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );
		
		$this->start_controls_tabs(
			'style_tabs'
		);

		$this->start_controls_tab(
			'style_normal_tab',
			[
				'label' => esc_html__( 'Normal', 'text-shuffle-widget' ),
			]
		);

        // Add typography control
        $this->add_group_control(
            \Elementor\Group_Control_Typography::get_type(),
            [
                'name' => 'typography',
                'label' => __('Typography', 'text-shuffle-widget'),
                'selector' => '{{WRAPPER}} .ar-shuffletext',
            ]
        );

        // Add text color control
        $this->add_control(
            'text_color',
            [
                'label' => __('Text Color', 'text-shuffle-widget'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Add background color control
        $this->add_control(
            'background_color',
            [
                'label' => __('Background Color', 'text-shuffle-widget'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        // Add padding control
        $this->add_responsive_control(
            'padding',
            [
                'label' => __('Padding', 'text-shuffle-widget'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Add margin control
        $this->add_responsive_control(
            'margin',
            [
                'label' => __('Margin', 'text-shuffle-widget'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Add border control
		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'border',
				'selector' => '{{WRAPPER}} .ar-shuffletext-container',
			]
		);
		
        // Add border radius control
        $this->add_responsive_control(
            'border_radius',
            [
                'label' => __('Border Radius', 'text-shuffle-widget'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
		
		$this->end_controls_tab();

		$this->start_controls_tab(
			'style_triggered_tab',
			[
				'label' => esc_html__( 'Triggered', 'text-shuffle-widget' ),
			]
		);

        // Add typography control
        $this->add_group_control(
            \Elementor\Group_Control_Typography::get_type(),
            [
                'name' => 'triggered_typography',
                'label' => __('Typography', 'text-shuffle-widget'),
                'selector' => '{{WRAPPER}} .ar-shuffletext.ar_shuffle_triggered',
            ]
        );

        // Add text color control
        $this->add_control(
            'triggered_text_color',
            [
                'label' => __('Text Color', 'text-shuffle-widget'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext.ar_shuffle_triggered' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Add background color control
        $this->add_control(
            'triggered_background_color',
            [
                'label' => __('Background Color', 'text-shuffle-widget'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container.ar_shuffle_triggered' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        // Add padding control
        $this->add_responsive_control(
            'triggered_padding',
            [
                'label' => __('Padding', 'text-shuffle-widget'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container.ar_shuffle_triggered' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Add margin control
        $this->add_responsive_control(
            'triggered_margin',
            [
                'label' => __('Margin', 'text-shuffle-widget'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container.ar_shuffle_triggered' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Add border control
		$this->add_group_control(
			\Elementor\Group_Control_Border::get_type(),
			[
				'name' => 'triggered_border',
				'selector' => '{{WRAPPER}} .ar-shuffletext-container.ar_shuffle_triggered',
			]
		);
		
        // Add border radius control
        $this->add_responsive_control(
            'triggered_border_radius',
            [
                'label' => __('Border Radius', 'text-shuffle-widget'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors' => [
                    '{{WRAPPER}} .ar-shuffletext-container.ar_shuffle_triggered' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
		
		$this->end_controls_tab();

		$this->end_controls_tabs();

        $this->end_controls_section();
    }

    protected function render() {
        $settings = $this->get_settings_for_display();
        echo do_shortcode('[ar_shuffletext text="' . $settings['text'] . '" element="' . $settings['element'] . '" counter="' . $settings['counter'] . '" timeout="' . $settings['timeout'] . '" trigger_class="' . $settings['trigger_class'] . '" trigger_distance="' . $settings['trigger_distance'] . '"]');
    }
}
