<?php
/*
Plugin Name: Proximity Text Shuffle
Description: A WordPress plugin for shuffling text.
Version: 1.0
Author: Harm Jakob Tolsma
*/
class Text_Shuffle_Shortcode {
    
	/**
	 * Add shortcode an make sure we load our scripts
	 */
	public function __construct() {
        add_shortcode('ar_shuffletext', array($this, 'shortcode_callback'));
        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
        add_action('elementor/widgets/widgets_registered', array($this, 'register_elementor_widget'));
    }

	/**
	 * Generate output for the shortcode
	 *
	 * @param	array	$atts		Array of shortcode attributes
	 * @return	string				The output for the shortcode
	 */
    public function shortcode_callback($atts) {
		// Parse and sanitize attributes
        $atts = shortcode_atts(array(
            'text' => '',
			'trigger_class' => '',
            'element' => 'span',
            'counter' => 3,
            'timeout' => 80,
			'trigger_distance' => 50,
        ), $atts);

        // Generate HTML output with unique ID
        $output = sprintf('<%s class="ar-shuffletext">%s</%s>', esc_attr($atts['element']), esc_html($atts['text']), esc_attr($atts['element']));

        // Return output with attributes as data attributes
        return sprintf('<div class="ar-shuffletext-container" data-counter="%d" data-timeout="%d" data-trigger-class="%s" data-trigger-distance="%d">%s</div>', esc_attr($atts['counter']), esc_attr($atts['timeout']), esc_attr($atts['trigger_class']), esc_attr($atts['trigger_distance']), $output);
    }

	/**
	 * Make sure the scripts will be loaded
	 */
    public function enqueue_scripts() {

        wp_enqueue_script('nearby-jquery', plugin_dir_url(__FILE__) . 'nearby.jquery.js', array('jquery'), '1.0', true);
        wp_enqueue_script('textshuffle-jquery', plugin_dir_url(__FILE__) . 'textshuffle.jquery.js', array('jquery'), '1.0', true);

        // Pass unique ID and trigger distance to JavaScript for initialization using wp_add_inline_script
		// Note that we still have to enqueue the script for the inline script to work.
		wp_enqueue_script('textshuffle-main', plugin_dir_url(__FILE__) . 'text-shuffle-dummy.js', array('jquery'), null, true );
        $inline_script = "
			(function($) {
                $(document).ready(function() {
					// Initialize textShuffle and nearby plugins for each shortcode instance
					$('.ar-shuffletext').each( function() {
						let shuffleElement = $(this);
						let triggerDistance = parseInt($(this).closest('.ar-shuffletext-container').data('trigger-distance'));
						let triggerTimeOut = parseInt($(this).closest('.ar-shuffletext-container').data('timeout'));
						let triggerCounter = parseInt($(this).closest('.ar-shuffletext-container').data('counter'));
						let triggerClass = $(this).closest('.ar-shuffletext-container').data('trigger-class');

						shuffleElement.textShuffle({
							counter: triggerCounter,
							timeout: triggerTimeOut
						});

						shuffleElement.nearby({
							onProgress: function(distance) {
								if (distance < triggerDistance) {
									if( !shuffleElement.hasClass('ar_shuffle_triggered') ) {
										shuffleElement.addClass('ar_shuffle_triggered');
										shuffleElement.closest('.ar-shuffletext-container').addClass('ar_shuffle_triggered');
										shuffleElement.addClass(triggerClass);
										shuffleElement.triggerTextShuffle();
									}
								}
								else {
									shuffleElement.removeClass('ar_shuffle_triggered');
									shuffleElement.closest('.ar-shuffletext-container').removeClass('ar_shuffle_triggered');
									shuffleElement.removeClass(triggerClass);
								}
							}
						});
					});
                });
            })(jQuery);
        ";

        wp_add_inline_script('textshuffle-main', $inline_script);
    }
	
	/**
	 * Register the Elementor widget
	 */
	public function register_elementor_widget() {
        if (class_exists('Elementor\Widget_Base')) {
            // Include the widget class file if necessary
            include_once('Text_Shuffle_Widget.php');

            // Register the widget
            \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Text_Shuffle_Widget());
        }
    }
}

// Initialize the plugin
$text_shuffle_shortcode = new Text_Shuffle_Shortcode();
