/**
 * TextShuffle jQuery plugin
 * Loosely adapted from the clip-path hover effect demo (https://tympanus.net/Tutorials/ClipHoverEffect/) from codrops (http://www.codrops.com)
  *
 * USAGE:
 *
 * Initalize using
 * 		$('#element').textShuffle({
 * 		    lettersAndSymbols: ['a', 'b', 'c'],
 * 		    counter: 5,
 * 		    timeout: 100
 * 		});
 *
 * Whenever you want to trigger the effect, call
 *		$('#element').triggerTextShuffle();
 */
(function($) {
    // Define the plugin function
    $.fn.textShuffle = function(options) {
        // Default options
        var settings = $.extend({
            lettersAndSymbols: ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '!', '@', '#', '$', '%', '^', '&', '*', '-', '_', '+', '=', ';', ':', '<', '>', ','],
            counter: 3,
            timeout: 80
        }, options);

		if( isNaN( settings.counter ) ) {
			settings.counter = 5;
		}
		if( isNaN( settings.timeout ) ) {
			settings.counter = 80;
		}

console.log( 'Initializing shuffle with counter: ' + settings.counter );
		

        // Define the splitText function
        const splitText = function(textElm) {
            let elmInnerText = textElm.text();
            let newInnerHTML = '';
            for (let n = 0; n < elmInnerText.length; n++) {
                newInnerHTML += '<span class="char" data-original-char="' + elmInnerText[n] + '">' + elmInnerText[n] + '</span>';
            }
            textElm.html(newInnerHTML);
        };

        // Define the shuffleChars function
        const shuffleChars = function(textElm) {
            let counter = textElm.data('counter') || 0;
            let maxcount = textElm.data('maxcounter') || 10;
            let timeout = textElm.data('timeout') || 10;

            textElm.find('.char').each(function() {
                let randomchar = settings.lettersAndSymbols[Math.floor(Math.random() * settings.lettersAndSymbols.length)];
                $(this).html(randomchar);
            });

            counter++;
            textElm.data('counter', counter);

            if (counter >= maxcount) {
                setTimeout(function() {
                    restoreChars(textElm);
                }, timeout);
            } else {
                setTimeout(function() {
                    shuffleChars(textElm);
                }, timeout);
            }
        };

        // Define the restoreChars function
        const restoreChars = function(textElm) {
            textElm.find('.char').each(function() {
                $(this).html($(this).attr("data-original-char"));
            });
            textElm.removeData('counter');
        };

		// Trigger shuffleChars externally
        $.fn.triggerTextShuffle = function() {
            return this.each(function() {
                shuffleChars($(this));
            });
        };
		
        // Iterate over each element in the jQuery object
        return this.each(function() {
            // Store a reference to the current element
            var $this = $(this);

            // Call the splitText function
            splitText($this);

			$this.data('maxcounter', settings.counter);
			$this.data('timeout', settings.timeout);
        });
    };
})(jQuery);
